#! d:\perl\bin\perl.exe

use strict;
use Win32::OLE qw(in);
use Getopt::Long;
my %config = ();
Getopt::Long::Configure("prefix_pattern=(-|\/)");
GetOptions(\%config, qw(server|s=s user|u=s passwd|p=s verbose|v file|f=s help|?|h));

if ($config{help} || (! %config)) {
	\_syntax();
	exit 1;
}

$config{user} = "" unless ($config{user});
$config{passwd} = "" unless ($config{passwd});

my $starttime = time();

my @servers;

if ($config{file}) {
	@servers = _readFile($config{file});
}
elsif ($config{server}) {
	push(@servers,$config{server});
}
else {
	print STDERR "You must enter either a system name, or a file to read system names from.\n";
	print "\n";
	\_syntax();
	exit 1;
}

foreach my $server (@servers) {
	print STDERR "Accessing $server...\n" if ($config{verbose});
	my $name;
	my $locatorObj = Win32::OLE->new('WbemScripting.SWbemLocator') || die 
		"Error creating locator object: ".Win32::OLE->LastError()."\n";
	$locatorObj->{Security_}->{impersonationlevel} = 3;
	my $serverObj = $locatorObj->ConnectServer($server,'root\cimv2',$config{user},$config{passwd}) 
		|| die "Error connecting to $server: ".Win32::OLE->LastError()."\n";

	foreach my $os (in $serverObj->InstancesOf("Win32_OperatingSystem")) {
		$name = $os->{CSName};
		print "Checking $name...\n" if ($config{verbose});
		print "Getting operating system info...\n" if ($config{verbose}); 
		my $file = "$name-os\.dat";
		open(FH,">$file") || die "Could not open $file for writing: $!\n";
		print FH "Name                   : ".$os->{Name}."\n";
		print FH "CSName                 : ".$os->{CSName}."\n";
		print FH "Version                : ".$os->{Version}."\n";
		print FH "CSDVersion             : ".$os->{CSDVersion}."\n";
		print FH "ServicePackMajorVersion: ".$os->{ServicePackMajorVersion}."\n";
		print FH "ServicePackMinorVersion: ".$os->{ServicePackMinorVersion}."\n";
		print FH "BuildNumber            : ".$os->{BuildNumber}."\n";
		print FH "BuildType              : ".$os->{BuildNumber}."\n";
		print FH "BootDevice             : ".$os->{BootDevice}."\n";
		print FH "WindowsDirectory       : ".$os->{WindowsDirectory}."\n";
		print FH "SystemDirectory        : ".$os->{SystemDirectory}."\n";
		print FH "SerialNumber           : ".$os->{SerialNumber}."\n";
		print FH "LastBootUpTime         : "._massageTime($os->{LastBootUpTime})."\n";
		print FH "LocalDateTime          : "._massageTime($os->{LocalDateTime})."\n";
		close(FH);
	}

	print "Getting process information...\n" if ($config{verbose});
	open(FH,">$name-services\.csv") || die "Could not open $name-services\.csv for writing: $!\n";
	print FH "Name,DisplayName,ProcessID,PathName,ServiceType,StartName,State,StartMode\n";
	foreach my $service (in $serverObj->InstancesOf("Win32_Service")) {
		print FH $service->{Name}.",".$service->{DisplayName}.",".$service->{ProcessId}.",".
        		 $service->{PathName}.",".$service->{ServiceType}.",".$service->{StartName}.",".
        		 $service->{State}.",".$service->{StartMode}."\n";
	}
	close(FH);

	print "Getting service information...\n" if ($config{verbose});
	open(FH,">$name-processes\.csv") || die "Could not open $name-processes\.csv for writing: $!\n";
# CommandLine is only available to XP and above
	print FH "Name,ProcessID,ParentProcessID,ExecutablePath,Handle\n";
	foreach my $proc (in $serverObj->InstancesOf("Win32_Process")) {
		print FH $proc->{Name}.",".$proc->{ProcessID}.",".$proc->{ParentProcessID}.",".$proc->{ExecutablePath}.
		         $proc->{Handle}."\n";
	}
	close(FH);
}

my $endtime = time();
print STDERR "Elapsed time: "._elapsedtime($endtime,$starttime)."\n" if ($config{verbose});

sub _massageTime {
	my $mtime = $_[0];
	
	my $year  = substr($mtime,0,4);
	my $month = substr($mtime,4,2);
	my $day   = substr($mtime,6,2);
	my $hour  = substr($mtime,8,2);
	my $min   = substr($mtime,10,2);
	my $sec   = substr($mtime,12,5);
	return "$month/$day/$year $hour:$min:$sec";
}

sub _elapsedtime {
	return ($_[0] - $_[1])." sec"; 
}

sub _readFile{
	my $file = $_[0];
	my @systems;
	
  if (-e $file) {
  	open(FH,$file) || die "Could not open $file: $!\n";
  	while(<FH>) {
  		chomp;
  		push(@systems,$_);
  	}
  	close(FH);
  }
  else {
  	print STDERR "$file not found.\n";
  	exit 1;
  }
  return @systems;
}

sub _syntax {
	my $script = ($0 =~ /([^\\]*?)$/);
	my $space  = " " x length($script);
	print<< "EOT";
WinData [-s system] [-u username] [-p password] [-v] [-f] [-h]
Collect system, process, and service information from remote Windows systems.
Output is stored in files <name>-[system|process|service].[dat|csv]
  -s system......Name of the system to scan
  -u username....Username used to connect to the remote system (usually
	               an Administrator)
  -p password....Password used to connect to the remote system
  -v.............Verbose output (verbose output goes to STDERR)
  -f file........File to read a list of machine names or IP addresses from
  -h.............Help (print this information)
		
copyright 2005 H. Carvey
EOT
}
